@@ -1,7 +1,7 @@
 #
 # backend_iptables.py: iptables backend for ufw
 #
-# Copyright 2008-2009 Canonical Ltd.
+# Copyright 2008-2010 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -18,8 +18,11 @@
 
 import os
 import re
+import shutil
+import stat
 import sys
 import tempfile
+import time
 
 from ufw.common import UFWError, UFWRule, config_dir, state_dir, prefix_dir
 from ufw.util import warn, debug, msg, cmd, cmd_pipe
@@ -165,7 +168,7 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
 
         return rstr
 
-    def get_running_raw(self):
+    def get_running_raw(self, set):
         '''Show current running status of firewall'''
         if self.dryrun:
             out = "> " + _("Checking raw iptables\n")
@@ -174,21 +177,82 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
 
         err_msg = _("problem running")
 
-        out = "IPV4:\n"
-        for table in ['filter', 'nat', 'mangle', 'raw']:
-            (rc, tmp) = cmd([self.iptables, '-L', '-n', '-v', '-x', '-t', \
-                             table])
+        args = ['-n', '-v', '-x', '-L']
+        items = []
+        items6 = []
+
+        if set == "raw":
+            args.append('-t')
+            items = ['filter', 'nat', 'mangle', 'raw']
+            items6 = ['filter', 'mangle', 'raw']
+        elif set == "builtins":
+            for c in ['INPUT', 'FORWARD', 'OUTPUT']:
+                items.append('filter:%s' % c)
+                items6.append('filter:%s' % c)
+            for c in ['PREROUTING', 'INPUT', 'FORWARD', 'OUTPUT', \
+                      'POSTROUTING']:
+                items.append('mangle:%s' % c)
+                items6.append('mangle:%s' % c)
+            for c in ['PREROUTING', 'OUTPUT']:
+                items.append('raw:%s' % c)
+                items6.append('raw:%s' % c)
+            for c in ['PREROUTING', 'POSTROUTING', 'OUTPUT']:
+                items.append('nat:%s' % c)
+        elif set == "before":
+            for b in ['input', 'forward', 'output']:
+                items.append('ufw-before-%s' % b)
+                items6.append('ufw6-before-%s' % b)
+        elif set == "user":
+            for b in ['input', 'forward', 'output']:
+                items.append('ufw-user-%s' % b)
+                items6.append('ufw6-user-%s' % b)
+            items.append('ufw-user-limit-accept')
+            items.append('ufw-user-limit')
+        elif set == "after":
+            for b in ['input', 'forward', 'output']:
+                items.append('ufw-after-%s' % b)
+                items6.append('ufw6-after-%s' % b)
+        elif set == "logging":
+            for b in ['input', 'forward', 'output']:
+                items.append('ufw-before-logging-%s' % b)
+                items6.append('ufw6-before-logging-%s' % b)
+                items.append('ufw-user-logging-%s' % b)
+                items6.append('ufw6-user-logging-%s' % b)
+                items.append('ufw-after-logging-%s' % b)
+                items6.append('ufw6-after-logging-%s' % b)
+            items.append('ufw-logging-allow')
+            items.append('ufw-logging-deny')
+            items6.append('ufw6-logging-allow')
+            items6.append('ufw6-logging-deny')
+
+        out = "IPV4 (%s):\n" % (set)
+        for i in items:
+            if ':' in i:
+                (t, c) = i.split(':')
+                out += "(%s) " % (t)
+                (rc, tmp) = cmd([self.iptables] + args + [c, '-t', t])
+            else:
+                (rc, tmp) = cmd([self.iptables] + args + [i])
             out += tmp
+            if set != "raw":
+                out += "\n"
             if rc != 0:
                 raise UFWError(out)
 
-        out += "\n\nIPV6:\n"
-        for table in ['filter', 'mangle', 'raw']:
-            (rc, tmp) = cmd([self.ip6tables, '-L', '-n', '-v', '-x', '-t', \
-                             table])
-            out += tmp
-            if rc != 0:
-                raise UFWError(out)
+        if set == "raw" or self.use_ipv6():
+            out += "\n\nIPV6:\n"
+            for i in items6:
+                if ':' in i:
+                    (t, c) = i.split(':')
+                    out += "(%s) " % (t)
+                    (rc, tmp) = cmd([self.iptables] + args + [c, '-t', t])
+                else:
+                    (rc, tmp) = cmd([self.ip6tables] + args + [i])
+                out += tmp
+                if set != "raw":
+                    out += "\n"
+                if rc != 0:
+                    raise UFWError(out)
 
         return out
 
@@ -347,13 +411,17 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
             str_to = _("To")
             str_from = _("From")
             str_action = _("Action")
-            rules_header = "%-26s %-12s%s\n" % (str_to, str_action, str_from)
+            rules_header = "%-26s %-12s%s\n" % \
+                            (str_to.decode("utf-8", 'ignore'), \
+                             str_action.decode("utf-8", 'ignore'), \
+                             str_from.decode("utf-8", 'ignore'))
             if show_count:
                 rules_header += "     "
-            rules_header += "%-26s %-12s%s\n" % ("-" * len(str_to), \
-                                                 "-" * len(str_action), \
-                                                 "-" * len(str_from))
-            full_str += rules_header
+            rules_header += "%-26s %-12s%s\n" % \
+                            ("-" * len(str_to.decode("utf-8", 'ignore')), \
+                             "-" * len(str_action.decode("utf-8", 'ignore')), \
+                             "-" * len(str_from.decode("utf-8", 'ignore')))
+            full_str += rules_header.encode('utf-8', 'ignore')
             if str != "":
                 full_str += str
             if str != "" and str_out != "":
@@ -402,13 +470,13 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
                 # Add the loglevel if not valid
                 try:
                     self.set_loglevel("low")
-                except:
+                except Exception:
                     err_msg = _("Could not set LOGLEVEL")
                     raise UFWError(err_msg)
             else:
                 try:
                     self.update_logging(self.defaults['loglevel'])
-                except:
+                except Exception:
                     err_msg = _("Could not load logging rules")
                     raise UFWError(err_msg)
 
@@ -447,7 +515,7 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
                 for c in self.chains['user']:
                     self._chain_cmd(c, ['-F', c])
                     self._chain_cmd(c, ['-Z', c])
-            except:
+            except Exception:
                 raise UFWError(err_msg)
 
             # then restore the system rules
@@ -948,7 +1016,7 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
                         c = pat_log.sub(r'\2', " ".join(s))
                         (rc, out) = cmd([exe, '-D', c, '-j', 'RETURN'])
                         if rc != 0:
-                            debug("FAILOK: " + err_msg)
+                            debug("FAILOK: -D %s -j RETURN" % (c))
 
         return rstr
 
@@ -1021,7 +1089,7 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
            self.chains['after'] + self.chains['misc']:
             try:
                 self._chain_cmd(c, ['-L', c, '-n'])
-            except:
+            except Exception:
                 raise UFWError(err_msg)
 
         # Flush all the logging chains except 'user'
@@ -1030,7 +1098,7 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
                self.chains['misc']:
                 self._chain_cmd(c, ['-F', c])
                 self._chain_cmd(c, ['-Z', c])
-        except:
+        except Exception:
             raise UFWError(err_msg)
 
         # Add logging rules to running firewall
@@ -1137,3 +1205,59 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
                                     '--log-prefix', msg] + largs, ''])
 
         return rules_t
+
+    def reset(self):
+        '''Reset the firewall'''
+        res = ""
+        # First make sure we have all the original files
+        all = []
+        for i in self.files:
+            if not self.files[i].endswith('.rules'):
+                continue
+            all.append(self.files[i])
+            fn = os.path.join(ufw.common.share_dir, "iptables", \
+                              os.path.basename(self.files[i]))
+            if not os.path.isfile(fn):
+                err_msg = _("Could not find '%s'. Aborting") % (fn)
+                raise UFWError(err_msg)
+            
+        ext = time.strftime("%Y%m%d_%H%M%S")
+
+	# This implementation will intentionally traceback if someone tries to
+        # do something to take advantage of the race conditions here.
+
+        # Don't do anything if the files already exist
+        for i in all:
+            fn = "%s.%s" % (i, ext)
+            if os.path.exists(fn):
+                err_msg = _("'%s' already exists. Aborting") % (fn)
+                raise UFWError(err_msg)
+
+        # Move the old to the new
+        for i in all:
+            fn = "%s.%s" % (i, ext)
+            res += _("Backing up '%s' to '%s'\n") % (os.path.basename(i), fn)
+            os.rename(i, fn)
+
+        # Copy files into place
+        for i in all:
+            old = "%s.%s" % (i, ext)
+            shutil.copy(os.path.join(ufw.common.share_dir, "iptables", \
+                                     os.path.basename(i)), \
+                        os.path.dirname(i))
+            shutil.copymode(old, i)
+
+            try:
+                statinfo = os.stat(i)
+                mode = statinfo[stat.ST_MODE]
+            except Exception:
+                warn_msg = _("Couldn't stat '%s'") % (i)
+                warn(warn_msg)
+                continue
+
+            if mode & stat.S_IWOTH:
+                res += _("WARN: '%s' is world writable") % (i)
+            elif mode & stat.S_IROTH:
+                res += _("'WARN: %s' is world readable") % (i)
+
+        return res
